/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { TestableComponentInterface } from "@wso2is/core/models";
import { EncodeDecodeUtils } from "@wso2is/core/utils";
import { CodeEditor, Text } from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, ReactNode, useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import { Divider, DropdownItemProps, Form, Icon } from "semantic-ui-react";
import {
    angularSDKInitialisationCode,
    angularSDKIntegrationCode, javascriptSDKInitialisationCode,
    javascriptSDKIntegrationCode, reactSDKInitialisationCode,
    reactSDKIntegrationCode
} from "./code-blocks";
import { SDKMeta } from "./meta";
import { SupportedTechnologyTypes } from "./models";
import {
    ApplicationInterface,
    ApplicationTemplateInterface
} from "../../../../features/applications";
import { AppState, store } from "../../../../features/core";
import { Config } from "../../../../features/core/configs";
import { VerticalStepper } from "../../../components/component-extensions/application/vertical-stepper";

interface IntegrateSDKsPropsInterface extends TestableComponentInterface {
    application: ApplicationInterface;
    technology: SupportedTechnologyTypes;
    template: ApplicationTemplateInterface;
    inboundProtocolConfig: any;
}

/**
 * Integrate SDKs to Single Page Applications.
 *
 * @param {IntegrateSDKsPropsInterface} props - Props injected into the component.
 * @return {React.ReactElement}
 */
export const IntegrateSDKs: FunctionComponent<IntegrateSDKsPropsInterface> = (
    props: IntegrateSDKsPropsInterface
): ReactElement => {

    const {
        inboundProtocolConfig,
        technology,
        [ "data-testid" ]: testId
    } = props;

    const [ SDKInitConfig, setSDKInitConfig ] = useState(undefined);
    const [ configuredCallbacks, setConfiguredCallbacks ] = useState<DropdownItemProps[]>([]);
    const [ selectedLoginCallBack, setSelectedLoginCallBack ] = useState<any>("");
    const [ selectedLogoutCallBack, setSelectedLogoutCallBack ] = useState<any>("");

    useEffect(() => {

        if (!inboundProtocolConfig?.oidc?.callbackURLs
            || !Array.isArray(inboundProtocolConfig.oidc.callbackURLs)
            || inboundProtocolConfig.oidc.callbackURLs.length < 1) {

            return;
        }

        const callbacks: string[] = EncodeDecodeUtils.decodeURLRegex(inboundProtocolConfig.oidc.callbackURLs[ 0 ]);

        if (callbacks && Array.isArray(callbacks) && callbacks.length > 1) {
            callbacks.forEach((url: string) => {
                setConfiguredCallbacks((prevItems) => [
                    ...prevItems,
                    {
                        key: url,
                        text: url,
                        value: url
                    }
                ]);
            });
        }
    }, [ inboundProtocolConfig ]);

    useEffect(() => {
        if (!inboundProtocolConfig?.oidc) {
            return;
        }

        const configs = {
            clientID: inboundProtocolConfig.oidc.clientId,
            serverOrigin: Config.getDeploymentConfig().serverHost,
            signInRedirectURL: (configuredCallbacks.length > 1)
                ? selectedLoginCallBack
                : inboundProtocolConfig.oidc.callbackURLs[ 0 ],
            signOutRedirectURL: (configuredCallbacks.length > 1)
                ? selectedLogoutCallBack
                : inboundProtocolConfig.oidc.callbackURLs[ 0 ]
        };

        setSDKInitConfig(configs);
    }, [ inboundProtocolConfig, configuredCallbacks, selectedLoginCallBack, selectedLogoutCallBack ]);

    const generateConfigureStep = (technology: SupportedTechnologyTypes): ReactNode => {

        if (technology === SupportedTechnologyTypes.REACT) {
            return (
                <>
                    <p>
                        Run below command to install the React SDK from the npm registry.
                    </p>
                    <CodeEditor
                        oneLiner
                        readOnly="nocursor"
                        withClipboardCopy
                        language="htmlmixed"
                        sourceCode={ SDKMeta.react.npmInstallCommand }
                    />
                    <div className="mt-3">
                        <p>
                            To learn more about API references, advanced usages etc., click on <a
                                href={ SDKMeta.react.readme }
                                target="_blank"
                                rel="noopener noreferrer"
                                className="link external"
                            >
                                <Icon name="github" />this link
                            </a> to navigate to documentation.
                        </p>
                    </div>
                </>
            );
        }

        if (technology === SupportedTechnologyTypes.JAVASCRIPT) {
            return (
                <>
                    <p>Add the following script in to the index.html of your application.</p>
                    <CodeEditor
                        oneLiner
                        readOnly="nocursor"
                        withClipboardCopy
                        language="htmlmixed"
                        sourceCode={ `<script src="${ SDKMeta.javascript.cdn }"></script>` }
                    />
                    <div className="mt-3">
                        <p>
                            To learn more about API references, advanced usages etc., click on <a
                            href={ SDKMeta.react.readme }
                            target="_blank"
                            rel="noopener noreferrer"
                            className="link external"
                        >
                            <Icon name="github" />this link
                        </a> to navigate to documentation.
                        </p>
                    </div>
                </>
            );
        }

        if (technology === SupportedTechnologyTypes.ANGULAR) {
            return (
                <>
                    <p>
                        Run below command to install the Angular SDK from npm registry.
                    </p>
                    <CodeEditor
                        oneLiner
                        readOnly="nocursor"
                        withClipboardCopy
                        language="htmlmixed"
                        sourceCode={ SDKMeta.angular.npmInstallCommand }
                    />
                    <div className="mt-3">
                        <p>
                            To learn more about API references, advanced usages etc., click on <a
                            href={ SDKMeta.react.readme }
                            target="_blank"
                            rel="noopener noreferrer"
                            className="link external"
                        >
                            <Icon name="github" />this link
                        </a> to navigate to documentation.
                        </p>
                    </div>
                </>
            );
        }

        return null;
    };

    const generateInitialisationStepTitle = (technology: SupportedTechnologyTypes): ReactNode => {

        if (technology === SupportedTechnologyTypes.REACT) {
            return "Configure AuthProvider";
        }

        if (technology === SupportedTechnologyTypes.ANGULAR) {
            return "Configure AuthModule";
        }

        if (technology === SupportedTechnologyTypes.JAVASCRIPT) {
            return "Configure AuthClient";
        }

        return "Configure Client";
    };

    const generateInitialisationStep = (technology: SupportedTechnologyTypes): ReactNode => {

        if (!SDKInitConfig) {
            return null;
        }

        if (technology === SupportedTechnologyTypes.REACT) {

            return (
                <>
                    <Text>
                        Asgardeo React SDK provides hooks to easily authenticate your React applications.
                        Use <code className="inline-code">signIn()</code> function from <code className="inline-code">
                        useAuthContext</code> hook to implement a <strong>Login button</strong>.
                        Similarly <code className="inline-code">signOut()</code> function can be used to implement a
                        <strong> Logout Button</strong>.
                    </Text>

                    <Text className="mt-3">
                        If you want to access the authenticated user details, use the <code className="inline-code">
                        state</code> object  from <code className="inline-code">useAuthContext</code> hook.
                    </Text>

                    <Divider hidden />

                    { renderCallbackURLSelectionOptions() }

                    <div className="code-segment">
                        <CodeEditor
                            showLineNumbers
                            withClipboardCopy
                            language="typescript"
                            sourceCode={
                                reactSDKInitialisationCode({
                                    clientID: SDKInitConfig?.clientID,
                                    serverOrigin: SDKInitConfig?.serverOrigin,
                                    signInRedirectURL: (configuredCallbacks.length > 1)
                                        ? selectedLoginCallBack
                                        : SDKInitConfig?.signInRedirectURL,
                                    signOutRedirectURL: (configuredCallbacks.length > 1)
                                        ? selectedLogoutCallBack
                                        : SDKInitConfig?.signOutRedirectURL
                                })
                            }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="dark"
                            readOnly="nocursor"
                        />
                    </div>
                </>
            );
        }

        if (technology === SupportedTechnologyTypes.JAVASCRIPT) {
            return (
                <>
                    <Text>
                        To initialize the SDK, use the <code className="inline-code">getInstance()</code> function in
                        the SDK and pass in the required <code className="inline-code">clientID</code>,
                        <code className="inline-code">serverOrigin</code>, <code className="inline-code">
                        signInRedirectURL</code> & <code className="inline-code">signOutRedirectURL</code> to the
                        <code className="inline-code">auth.initialize()</code> function.
                    </Text>

                    <Divider hidden />

                    { renderCallbackURLSelectionOptions() }

                    <div className="code-segment">
                        <CodeEditor
                            showLineNumbers
                            withClipboardCopy
                            language="typescript"
                            sourceCode={
                                javascriptSDKInitialisationCode({
                                    clientID: SDKInitConfig?.clientID,
                                    serverOrigin: SDKInitConfig?.serverOrigin,
                                    signInRedirectURL: (configuredCallbacks.length > 1)
                                        ? selectedLoginCallBack
                                        : SDKInitConfig?.signInRedirectURL,
                                    signOutRedirectURL: (configuredCallbacks.length > 1)
                                        ? selectedLogoutCallBack
                                        : SDKInitConfig?.signOutRedirectURL
                                })
                            }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="dark"
                            readOnly="nocursor"
                        />
                    </div>
                </>
            );
        }

        if (technology === SupportedTechnologyTypes.ANGULAR) {
            return (
                <>
                    <Text>
                        Import the <code className="inline-code">AuthProvider</code> from React SDK and wrap your root
                        component.
                    </Text>

                    <Text className="mt-3">
                        You can initialize the SDK instance by passing the <code className="inline-code">
                        clientID</code>, <code className="inline-code">serverOrigin</code>,
                        <code className="inline-code">signInRedirectURL</code> & <code className="inline-code">
                        signOutRedirectURL</code> to the <code className="inline-code">config</code> object.
                    </Text>

                    <Divider hidden />

                    { renderCallbackURLSelectionOptions() }

                    <div className="code-segment">
                        <CodeEditor
                            showLineNumbers
                            withClipboardCopy
                            language="typescript"
                            sourceCode={
                                angularSDKInitialisationCode({
                                    clientID: SDKInitConfig?.clientID,
                                    serverOrigin: SDKInitConfig?.serverOrigin,
                                    signInRedirectURL: (configuredCallbacks.length > 1)
                                        ? selectedLoginCallBack
                                        : SDKInitConfig?.signInRedirectURL,
                                    signOutRedirectURL: (configuredCallbacks.length > 1)
                                        ? selectedLogoutCallBack
                                        : SDKInitConfig?.signOutRedirectURL
                                })
                            }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="dark"
                            readOnly="nocursor"
                        />
                    </div>
                </>
            );
        }

        return null;
    };

    const generateAuthenticationStep = (technology: SupportedTechnologyTypes): ReactNode => {

        if (technology === SupportedTechnologyTypes.REACT) {

            return (
                <>
                    <Text>
                        Asgardeo React SDK provides hooks to easily authenticate your React applications.
                        Use <code className="inline-code">signIn</code> function from <code className="inline-code">
                        useAuthContext</code> hook to implement a <strong>Login button</strong>.
                        Similarly <code className="inline-code">signOut</code> function can be used to implement a
                        <strong> Logout Button</strong>.
                    </Text>

                    <Text className="mt-3">
                        If you want to access the authenticated user details, use the <code className="inline-code">
                        state</code> object  from <code className="inline-code">useAuthContext</code> hook.
                    </Text>

                    <Divider hidden />

                    <div className="code-segment">
                        <CodeEditor
                            showLineNumbers
                            withClipboardCopy
                            language="javascript"
                            sourceCode={ reactSDKIntegrationCode() }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="dark"
                            readOnly="nocursor"
                        />
                    </div>
                </>
            );
        }

        if (technology === SupportedTechnologyTypes.ANGULAR) {

            return (
                <>
                    <Text>
                        Inject the <code className="inline-code">AsgardeoAuthService</code> in your components
                        to access the authenticated state and signIn and siginOut funtionalities.
                    </Text>

                    <Divider hidden />

                    <div className="code-segment">
                        <CodeEditor
                            showLineNumbers
                            withClipboardCopy
                            language="typescript"
                            sourceCode={ angularSDKIntegrationCode() }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="dark"
                            readOnly="nocursor"
                        />
                    </div>
                </>
            );
        }

        if (technology === SupportedTechnologyTypes.JAVASCRIPT) {

            return (
                <>
                    <Text>
                        The <code className="inline-code">sign-in</code> hook is used to fire a callback function
                        after signing in is successful.
                    </Text>

                    <Text className="mt-3">
                        To sign in, simply call the <code className="inline-code">signIn()</code> function using the
                        created instance. Similarly, call the <code className="inline-code">signOut()</code> function
                        in a sign out from the application.
                    </Text>

                    <Divider hidden />

                    <div className="code-segment">
                        <CodeEditor
                            showLineNumbers
                            withClipboardCopy
                            language="javascript"
                            sourceCode={ javascriptSDKIntegrationCode() }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="dark"
                            readOnly="nocursor"
                        />
                    </div>
                </>
            );
        }

        return null;
    };

    const renderCallbackURLSelectionOptions = () => {

        if (configuredCallbacks.length < 1) {
            return null;
        }

        return  (
            <>
                <Form>
                    <Form.Group widths="3">
                        <Form.Select
                            fluid
                            onChange={ (value, data) => {
                                setSelectedLoginCallBack(data.value);
                            } }
                            label="Select Sign-in Redirect URL:"
                            options={ configuredCallbacks }
                            placeholder="Sign-in Redirect URL"
                        />
                    </Form.Group>
                    <Form.Group widths="3">
                        <Form.Select
                            fluid
                            onChange={ (value, data) => {
                                setSelectedLogoutCallBack(data.value);
                            } }
                            label="Select Sign-out Redirect URL:"
                            options={ configuredCallbacks }
                            laceholder="Sign-out Redirect URL"
                        />
                    </Form.Group>
                </Form>
                <Divider hidden/>
            </>
        );
    };

    const steps = [
            {
                stepContent: generateConfigureStep(technology),
                stepTitle: "Install SDK"
            },
            {
                stepContent: generateInitialisationStep(technology),
                stepTitle: generateInitialisationStepTitle(technology)
            },
            {
                stepContent: generateAuthenticationStep(technology),
                stepTitle: "Add Login"
            }
        ]
    ;

    return (
        <VerticalStepper
            alwaysOpen
            isSidePanelOpen
            stepContent={ steps }
            isNextEnabled={ technology !== undefined }
        />
    );
};
