/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { AlertInterface, AlertLevels, TestableComponentInterface } from "@wso2is/core/models";
import { addAlert } from "@wso2is/core/store";
import { EncodeDecodeUtils } from "@wso2is/core/utils";
import { CodeEditor, GenericIcon, Heading, LinkButton } from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import { Button, Divider, Icon, Message } from "semantic-ui-react";
import { ReactComponent as AngularLogo } from "./assets/angular-logo.svg";
import DownloadIllustration from "./assets/config-copying-illustration.svg";
import { ReactComponent as JavaScriptLogo } from "./assets/javascript-logo.svg";
import { SDKMeta } from "./meta";
import { SupportedTechnologyTypes } from "./models";
import {
    ApplicationInterface,
    ApplicationManagementUtils,
    ApplicationTemplateInterface,
    OIDCDataInterface,
    SupportedAuthProtocolTypes,
    updateAuthProtocolConfig
} from "../../../../features/applications";
import { AppState } from "../../../../features/core";
import { Config } from "../../../../features/core/configs";
import { ReactComponent as DownloadArrow } from "../../../assets/images/icons/download-arrow.svg";
import ReactLogoDataURL from "../../../assets/images/icons/react-icon.svg";
import { VerticalStepper } from "../../../components/component-extensions/application/vertical-stepper";

interface TryoutSamplesPropsInterface extends TestableComponentInterface {
    application: ApplicationInterface;
    template: ApplicationTemplateInterface;
    technology: SupportedTechnologyTypes;
    inboundProtocolConfig: any;
}

/**
 * Tryout Samples of Single Page Applications.
 *
 * @param {TryoutSamplesPropsInterface} props - Props injected into the component.
 * @return {React.ReactElement}
 */
export const TryoutSamples: FunctionComponent<TryoutSamplesPropsInterface> = (
    props: TryoutSamplesPropsInterface
): ReactElement => {

    const {
        application,
        inboundProtocolConfig,
        technology,
        [ "data-testid" ]: testId
    } = props;

    const dispatch = useDispatch();

    const [ authConfig, setAuthConfig ] = useState(undefined);

    const [ callbacksUpdated, setCallbacksUpdated ] = useState<boolean>(false);

    useEffect(() => {
        if (inboundProtocolConfig == undefined) {
            return;
        }

        const configs = {
            clientID: inboundProtocolConfig.oidc.clientId,
            serverOrigin: Config.getDeploymentConfig().serverHost,
            signInRedirectURL: "https://localhost:5000",
            signOutRedirectURL: "https://localhost:5000"
        };

        setAuthConfig(configs);
    }, [ inboundProtocolConfig ]);

    const checkCallbacks = (urlToCheck: string) => {
        const urlArray = inboundProtocolConfig.oidc
            ? EncodeDecodeUtils.decodeURLRegex(inboundProtocolConfig.oidc.callbackURLs[ 0 ])
            : [];

        if (!urlArray || !Array.isArray(urlArray) || urlArray.length < 1) {

            return false;
        }

        return urlArray.includes(urlToCheck);
    };

    const generateIntegrateCode = (technology: SupportedTechnologyTypes) => {

        if (!authConfig) {
            return null;
        }

        if (technology === SupportedTechnologyTypes.REACT) {
            return `{
    "clientID": "${ authConfig.clientID }",
    "serverOrigin": "${ authConfig.serverOrigin }",
    "signInRedirectURL": "${ authConfig.signInRedirectURL }",
    "signOutRedirectURL": "${ authConfig.signOutRedirectURL }"
}`;
        }

        return `const authConfig = {
    clientID: "${ authConfig.clientID }",
    signInRedirectURL: "${ authConfig.signInRedirectURL }",
    signOutRedirectURL: "${ authConfig.signOutRedirectURL }",
    serverOrigin: "${ authConfig.serverOrigin }",
    enablePKCE: true
};`;

    };

    const downloadConfigFile = (): void => {
        const blob = new Blob(
            [ JSON.stringify(authConfig, null, 2) ],
            { type: "application/json" }
        );
        const url = window.URL.createObjectURL(blob);
        const a = document.createElement("a");
        a.style.display = "none";
        a.href = url;
        a.download = "config.json";
        document.body.appendChild(a);
        a.click();
        window.URL.revokeObjectURL(url);
    };

    const handleAddCallback = (url: string) => {

        const configuredCallbacks: any = ApplicationManagementUtils.buildCallBackURLWithSeparator(
            inboundProtocolConfig.oidc.callbackURLs[ 0 ]).split(",");

        const shouldUpdateCallbacks: boolean = !configuredCallbacks.includes(url);
        const shouldUpdateAllowedOrigins: boolean = !inboundProtocolConfig.oidc.allowedOrigins.includes(url);

        const body: OIDCDataInterface = {
            ...inboundProtocolConfig.oidc,
            allowedOrigins: shouldUpdateAllowedOrigins
                ? [ ...inboundProtocolConfig.oidc.allowedOrigins, url ]
                : inboundProtocolConfig.oidc.allowedOrigins,
            callbackURLs: shouldUpdateCallbacks
                ? [ ApplicationManagementUtils.buildCallBackUrlWithRegExp([ ...configuredCallbacks, url ].join(",")) ]
                : inboundProtocolConfig.oidc.callbackURLs
        };

        const resolveAlertContent = () => {

            if (shouldUpdateAllowedOrigins && shouldUpdateCallbacks) {
                return {
                    description: "Successfully updated the URLs in the application.",
                    level: AlertLevels.SUCCESS,
                    message: "Updated the URLs"
                };
            } else if (shouldUpdateCallbacks) {
                return {
                    description: "Successfully updated the Allowed Origins.",
                    level: AlertLevels.SUCCESS,
                    message: "Updated the URLs"
                };
            }

            return {
                description: "Successfully updated the Callback URLs in the application.",
                level: AlertLevels.SUCCESS,
                message: "Updated the URLs"
            };
        };

        updateAuthProtocolConfig(application.id, body, SupportedAuthProtocolTypes.OIDC)
            .then(() => {
                setCallbacksUpdated(true);

                dispatch(addAlert<AlertInterface>(resolveAlertContent()));
            })
            .catch(() => {
                dispatch(addAlert<AlertInterface>({
                    description: "An error occurred while updating the application.",
                    level: AlertLevels.ERROR,
                    message: "Error occurred"
                }));
            });
    };

    const renderSampleDownloadButton = (technology: SupportedTechnologyTypes) => {

        if (technology === SupportedTechnologyTypes.REACT) {
            return (
                <>
                    <p>Click on the button bellow to download the sample.</p>
                    <Button.Group className="sample-download-button">
                        <Button
                            data-testid={ `${ testId }-download-react-sample` }
                            className="download-section"
                            basic
                            onClick={ () => window.open(SDKMeta.javascript.samples.react.artifact, "") }
                        >
                            <GenericIcon
                                transparent
                                icon={ ReactLogoDataURL }
                                size="mini"
                                spaced="right"
                                floated="left"
                            />
                            Download { technology } Sample
                            <GenericIcon
                                transparent
                                icon={ DownloadArrow }
                                className="download-icon"
                                size="mini"
                                fill="default"
                                spaced="left"
                                floated="right"
                            />
                        </Button>
                        <Button
                            icon="github"
                            className="external-link-section"
                            onClick={ () => window.open(SDKMeta.javascript.samples.react.repository, "") }
                        />
                    </Button.Group>
                    <div className="mt-3">
                        <a
                            href={ SDKMeta.javascript.samples.react.repository }
                            target="_blank"
                            rel="noopener noreferrer"
                            className="link external"
                        >
                            Click here <Icon name="external"/>
                        </a> to explore more samples like this.
                    </div>
                </>
            );
        } else if (technology === SupportedTechnologyTypes.JAVASCRIPT) {
            return (
                <>
                    <p>Click on the button bellow to download the sample.</p>
                    <Button.Group className="sample-download-button">
                        <Button
                            data-testid={ `${ testId }-download-js-sample` }
                            className="download-section"
                            basic
                            onClick={ () => window.open(SDKMeta.javascript.samples.javascript.artifact, "") }
                        >
                            <GenericIcon
                                transparent
                                icon={ JavaScriptLogo }
                                size="mini"
                                spaced="right"
                                floated="left"
                            />
                            Download { technology } Sample
                            <GenericIcon
                                transparent
                                icon={ DownloadArrow }
                                className="download-icon"
                                size="mini"
                                spaced="left"
                                floated="right"
                            />
                        </Button>
                        <Button
                            icon="github"
                            className="external-link-section"
                            onClick={ () => window.open(SDKMeta.javascript.samples.javascript.repository, "") }
                        />
                    </Button.Group>
                    <div className="mt-3">
                        <a
                            href={ SDKMeta.javascript.samples.javascript.repository }
                            target="_blank"
                            rel="noopener noreferrer"
                            className="link external"
                        >
                            Click here <Icon name="external"/>
                        </a> to explore more samples like this.
                    </div>
                </>
            );
        } else if (technology === SupportedTechnologyTypes.ANGULAR) {
            return (
                <>
                    <p>Click on the button bellow to download the sample.</p>
                    <Button.Group className="sample-download-button">
                        <Button
                            data-testid={ `${ testId }-download-angular-sample` }
                            className="download-section"
                            basic
                            onClick={ () => window.open("", "") }
                        >
                            <GenericIcon
                                transparent
                                icon={ AngularLogo }
                                size="mini"
                                spaced="right"
                                floated="left"
                            />
                            Download { technology } Sample
                            <GenericIcon
                                transparent
                                icon={ DownloadArrow }
                                className="download-icon"
                                size="mini"
                                spaced="left"
                                floated="right"
                            />
                        </Button>
                        <Button
                            icon="github"
                            className="external-link-section"
                            onClick={ () => window.open(SDKMeta.javascript.samples.javascript.repository, "") }
                        />
                    </Button.Group>
                    <div className="mt-3">
                        <a
                            href={ SDKMeta.javascript.samples.javascript.repository }
                            target="_blank"
                            rel="noopener noreferrer"
                            className="link external"
                        >
                            Click here <Icon name="external"/>
                        </a> to explore more samples like this.
                    </div>
                </>
            );
        }
    };

    const generateConfigureStep = (technology: SupportedTechnologyTypes) => {

        if (technology === SupportedTechnologyTypes.REACT) {
            return (
                <>
                    <Heading as="h4" className="mt-4">
                        Copy the Configuration
                    </Heading>
                    <p>The following generated configuration will replace the <strong>src/config.json</strong> file.</p>
                    <div className="code-segment">
                        <CodeEditor
                            oneLiner
                            readOnly="nocursor"
                            withClipboardCopy
                            showLineNumbers
                            className="application-sample-config-editor"
                            language="json"
                            sourceCode={ generateIntegrateCode(SupportedTechnologyTypes.REACT) }
                            beautify={ true }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="dark"
                        />
                    </div>

                    <Heading as="h4">
                        Replace the default configuration
                    </Heading>
                    <p>Replace the <strong>src/config.json</strong> file with the copied configuration.</p>
                    <GenericIcon
                        icon={ DownloadIllustration }
                        size="large"
                        transparent
                        className="download-illustration"
                    />
                    <Divider hidden/>
                </>
            );
        } else if (technology === SupportedTechnologyTypes.JAVASCRIPT) {
            return (
                <>
                    <p className="mt-4">
                        Open the <strong>index.html</strong> file located at the root of
                        the project & scroll down to the <strong>{ "<script>" }</strong> tag at the end of the
                        <strong> body</strong> tag and find `authConfig` object.
                    </p>
                    <p>Change the configuration object as follows.</p>
                    <div className="code-segment">
                        <CodeEditor
                            oneLiner
                            readOnly="nocursor"
                            withClipboardCopy
                            showLineNumbers
                            className="application-sample-config-editor"
                            language="javascript"
                            sourceCode={ generateIntegrateCode(SupportedTechnologyTypes.JAVASCRIPT) }
                            beautify={ true }
                            theme="dark"
                            options={ {
                                lineWrapping: true
                            } }
                        />
                    </div>
                </>
            );
        } else if (technology === SupportedTechnologyTypes.ANGULAR) {
            return (
                <>
                    <Heading as="h4" className="mt-4">
                        Copy the Configuration
                    </Heading>
                    <p>The following generated configuration will replace the <strong>src/config.json</strong> file.</p>
                    <div className="code-segment">
                        <CodeEditor
                            oneLiner
                            readOnly="nocursor"
                            withClipboardCopy
                            showLineNumbers
                            className="application-sample-config-editor"
                            language="json"
                            sourceCode={ generateIntegrateCode(SupportedTechnologyTypes.ANGULAR) }
                            beautify={ true }
                            options={ {
                                lineWrapping: true
                            } }
                            theme="dark"
                        />
                    </div>

                    <Heading as="h4">
                        Replace the default Configuration
                    </Heading>
                    <p>Replace the <strong>src/config.json</strong> file with the copied configuration.</p>
                    <GenericIcon
                        icon={ DownloadIllustration }
                        size="large"
                        transparent
                        className="download-illustration"
                    />
                    <Divider hidden/>
                </>
            );
        }

        return null;
    };

    const webAppContent = [
        {
            stepContent: renderSampleDownloadButton(technology),
            stepTitle: "Download"
        },
        {
            stepContent: (
                <>
                    {
                        !callbacksUpdated
                        && !(checkCallbacks("https://localhost:5000")
                            && checkCallbacks("https://localhost:5000"))
                            ? (
                                <Message warning>
                                    <p>
                                        In order to try out the sample, you need to
                                        add <strong>https://localhost:5000</strong> to <strong>Allowed Redirect
                                        URLs</strong>
                                        <Button color="orange" floated="right"
                                                onClick={ () => handleAddCallback("https://localhost:5000") }>Add
                                            Now</Button>
                                    </p>
                                </Message>
                            )
                            : null
                    }
                    { generateConfigureStep(technology) }
                </>
            ),
            stepTitle: "Configure"
        },
        {
            stepContent: (
                <>
                    <p className="mt-2">
                        Run the below command at the root of the project to fire up the development server. The app
                        should open on <a>https://localhost:5000</a>.
                    </p>
                    <CodeEditor
                        oneLiner
                        readOnly="nocursor"
                        withClipboardCopy
                        language="javascript"
                        sourceCode="npm install && npm start"
                    />
                    <div className="mt-3">
                        For more details and troubleshooting, navigate to our <a
                            href={ SDKMeta.javascript.samples.react.repository }
                            target="_blank"
                            rel="noopener noreferrer"
                            className="link external"
                        >
                            <Icon name="github"/>Github repository
                        </a>
                    </div>
                </>
            ),
            stepTitle: "Run"
        }
    ];

    return (
        <VerticalStepper
            alwaysOpen
            isSidePanelOpen
            stepContent={ webAppContent }
            isNextEnabled={ technology !== undefined }
        />
    );
};
